﻿using SFML.Graphics;
using SFML.System;
using System;

//====================================================
// Описание работы классов и методов исходника на:
// www.interestprograms.ru
// Исходные коды программ и игр
// Автор исходных кодов Клочков Павел
//====================================================


namespace sfml4
{
    class Rocket : Sprite
    {
        // Скорость перемещения ракеты по направлению к цели.
        private readonly float scalarVelocity = 2.3f;

        // Флаг однократного срабатывания события взрыва.
        private bool oneEvent = true;

        // Позиция цели ракеты
        private Vector2f positionTo;

        // Нормализованный вектор направления.
        // Для вычисления вектора скорости
        private Vector2f normalDirection;

        // Базовый вектор от которого отсчитывается угол поворота ракеты.
        // Высчитывается всегда абсолютный угол.
        private readonly Vector2f vectorBaseAngle = new Vector2f(1, 0);

        // Событие момента взрыва.
        public event EventHandler ExplosionEvent;


        public Rocket()
        {
            // Картинка ракеты.
            this.Texture = new Texture(Resource.rocket);

            // Масштаб размеров ракеты относительно картинки.
            this.Scale = new Vector2f(0.2f, 0.2f);

            // Центр удара ракеты, активная точка.
            Origin = new Vector2f(GetLocalBounds().Width, GetLocalBounds().Height / 2);

            // Первоначальная позиция ракеты.
            Position = new Vector2f(150, 50);

        }

        // Вычисление направления к цели.
        public void ComputeDirection(Vector2f newPos)
        {
            // разрешаем только однократное событие.
            oneEvent = true;

            // Запоминаем новую позицию.
            positionTo = newPos;


            // Вычисляем вектор нового направления.
            Vector2f dotPos = Position;
            Vector2f dotTo = positionTo;
            Vector2f dir = dotTo - dotPos;


            // Вычисление вектора направления к цели.
            var temp = System.Numerics.Vector2.Normalize(new System.Numerics.Vector2(dir.X, dir.Y));
            normalDirection = new Vector2f(temp.X, temp.Y);


            // Получаем угол поворота ракеты к цели на новое направление, 
            // относительно базового вектора.
            float angle = AngleBetweenVectors(vectorBaseAngle, normalDirection);


            // Угол поворота всегда в направлении вектора скорости.
            // Поворот мгновенный, для упрощения, пренебрегая рациональным направлением вращения.
            this.Rotation = angle * Math.Sign(normalDirection.Y);

        }


        /// <summary>
        /// Рисуем спрайт ракеты вместе с анимацией движения.
        /// </summary>
        public Sprite Draw()
        {
            // Начальная и конечная позиция ракеты
            var start = new System.Numerics.Vector2(Position.X, Position.Y);
            var finish = new System.Numerics.Vector2(positionTo.X, positionTo.Y);

            // Расчет расстояния от текущей позиции ракеты до цели.
            float distance = System.Numerics.Vector2.Distance(start, finish);


            if (distance > (scalarVelocity + 0.1f))
            {
                // Умножая скалярную величину скорости на
                // единичный вектор направления получаем вектор скорости.
                Vector2f velocity = scalarVelocity * normalDirection;

                // Высчитываем новую позицию для каждого кадра.
                Position += velocity;
            }
            else
            {
                // Событие поражение цели.
                if (oneEvent == true)
                {
                    ExplosionEvent(this, null);

                    // Разрешаем только одно событие взрыва при поражении цели.
                    oneEvent = false;
                }
            }

            return this;
        }

        /// <summary>
        /// Вычисление угла между векторами
        /// </summary>
        /// <param name="normal1">нормализованный вектор</param>
        /// <param name="normal2">нормализованный вектор</param>
        /// <returns>угол в градусах</returns>
        private float AngleBetweenVectors(Vector2f normal1, Vector2f normal2)
        {
            // Угол между векторами в градусах.
            double scalar = Math.Round(normal1.X * normal2.X, 3) + Math.Round(normal1.Y * normal2.Y, 3);
            float rad = (float)Math.Acos(scalar);
            float degree = rad * 180 / (float)Math.PI;

            return degree;
        }
    }
}
